@@ -1,7 +1,7 @@
 from zeep.settings import Settings
 
 
-class XmlParserContext(object):
+class XmlParserContext:
     """Parser context when parsing XML elements"""
 
     def __init__(self, settings=None):
